package ch.ethz.matsim.projects.astra_2018_002;

import org.matsim.api.core.v01.Scenario;
import org.matsim.core.config.Config;
import org.matsim.core.config.ConfigUtils;
import org.matsim.core.controler.Controler;
import org.matsim.core.scenario.ScenarioUtils;

import ch.ethz.matsim.baseline_scenario.config.CommandLine;
import ch.ethz.matsim.baseline_scenario.config.CommandLine.ConfigurationException;
import ch.ethz.matsim.baseline_scenario.traffic.BaselineTrafficModule;
import ch.ethz.matsim.baseline_scenario.transit.BaselineTransitModule;
import ch.ethz.matsim.baseline_scenario.transit.routing.DefaultEnrichedTransitRoute;
import ch.ethz.matsim.baseline_scenario.transit.routing.DefaultEnrichedTransitRouteFactory;
import ch.ethz.matsim.projects.astra_2018_002.flow_efficiency.FlowEfficiencyConfigurator;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissDiscreteModeChoiceModule;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissModeChoiceConfigurator;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissUtilityParameters.UtilitySet;
import ch.ethz.matsim.projects.astra_2018_002.private_av.PrivateAVConfigurator;
import ch.ethz.matsim.projects.astra_2018_002.shared_av.SharedAVConfigurator;
import ch.sbb.matsim.routing.pt.raptor.SwissRailRaptorModule;

public class RunASTRA2018002 {
	static public void main(String[] args) throws ConfigurationException {
		CommandLine cmd = new CommandLine.Builder(args) //
				.requireOptions("config-path", "model") //
				.allowOptions("use-route-choice", "use-only-significant", "utility-parameters", "crossing-penalty") //
				.allowPrefixes(SwissDiscreteModeChoiceModule.COMMAND_LINE_PREFIX) //
				.build();

		// ASTRAConfigGroup astraConfig = new ASTRAConfigGroup();
		// astraConfig.setActiveServiceArea("zurich");
		// astraConfig.setPooledFleetSize(0);
		// astraConfig.setTaxiFleetSize(100);
		// astraConfig.setServiceAreaShapefile("gis/operating_areas.shp");
		// astraConfig.setServiceAreaAttribute("OAREA");
		// astraConfig.setWaitingTimeZoneShapefile("gis/waiting_time_areas.shp");
		// astraConfig.setWaitingTimeZoneIdentifierAttribute("WZONE");

		// Load config
		Config config = ConfigUtils.loadConfig(cmd.getOptionStrict("config-path"));

		// Adjust config (they need to stay in the correct order!)
		SwissModeChoiceConfigurator.configure(config);
		FlowEfficiencyConfigurator.configure(config);
		SharedAVConfigurator.configure(config);
		PrivateAVConfigurator.configure(config);
		cmd.applyConfiguration(config);

		// Load scenario
		Scenario scenario = ScenarioUtils.createScenario(config);
		scenario.getPopulation().getFactory().getRouteFactories().setRouteFactory(DefaultEnrichedTransitRoute.class,
				new DefaultEnrichedTransitRouteFactory());
		ScenarioUtils.loadScenario(scenario);

		// Adjust scenario
		SwissModeChoiceConfigurator.copyHousholdAttributes(scenario);
		FlowEfficiencyConfigurator.defineVehicleTypes(scenario);
		ASTRAConfigurator.adjustNetwork(scenario);

		// Set up controller
		Controler controller = new Controler(scenario);
		controller.addOverridingModule(new SwissRailRaptorModule());
		controller.addOverridingModule(new BaselineTransitModule());
		controller.addOverridingModule(new ASTRAModule());
		controller.addOverridingModule(
				new BaselineTrafficModule(cmd.getOption("crossing-penalty").map(Double::parseDouble).orElse(3.0)));
		SharedAVConfigurator.configureController(controller);

		UtilitySet utilitySet = Enum.valueOf(UtilitySet.class, cmd.getOptionStrict("model"));
		boolean useRouteChoice = cmd.getOption("use-route-choice").map(Boolean::parseBoolean).orElse(true);
		boolean useOnlySignificant = cmd.getOption("use-only-significant").map(Boolean::parseBoolean).orElse(false);
		controller.addOverridingModule(
				new SwissDiscreteModeChoiceModule(utilitySet, useRouteChoice, useOnlySignificant, cmd));

		// Run
		controller.run();
	}
}
