package ch.ethz.matsim.projects.astra_2018_002.mode_choice;

import java.io.File;
import java.io.IOException;

import org.matsim.api.core.v01.network.Network;
import org.matsim.core.router.TripRouter;
import org.matsim.facilities.ActivityFacilities;
import org.matsim.pt.transitSchedule.api.TransitSchedule;

import com.google.inject.Provides;
import com.google.inject.Singleton;

import ch.ethz.matsim.baseline_scenario.config.CommandLine;
import ch.ethz.matsim.baseline_scenario.config.CommandLine.ConfigurationException;
import ch.ethz.matsim.discrete_mode_choice.modules.AbstractDiscreteModeChoiceExtension;
import ch.ethz.matsim.discrete_mode_choice.modules.DiscreteModeChoiceModule;
import ch.ethz.matsim.discrete_mode_choice.modules.config.DiscreteModeChoiceConfigGroup;
import ch.ethz.matsim.discrete_mode_choice.modules.config.ModeAvailabilityConfigGroup;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.SwissUtilityParameters.UtilitySet;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.cost.CostModel;
import ch.ethz.matsim.projects.astra_2018_002.mode_choice.cost.CostParameters;

public class SwissDiscreteModeChoiceModule extends AbstractDiscreteModeChoiceExtension {
	public final static String COMMAND_LINE_PREFIX = "utility";
	public static final String SWISS_ESTIMATOR_NAME = "Swiss";
	public static final String SWISS_MODE_AVAILABILITY_NAME = "SwissModeAvailability";

	private final UtilitySet utilitySet;
	private final boolean useRouteChoice;
	private final boolean onlySignificant;

	private final CommandLine cmd;

	public SwissDiscreteModeChoiceModule(UtilitySet utilitySet, boolean useRouteChoice, boolean onlySignificant,
			CommandLine cmd) {
		this.utilitySet = utilitySet;
		this.useRouteChoice = useRouteChoice;
		this.onlySignificant = onlySignificant;
		this.cmd = cmd;
	}

	@Override
	protected void installExtension() {
		install(new DiscreteModeChoiceModule());
		bindTripEstimator(SWISS_ESTIMATOR_NAME).to(SwissUtilityEstimator.class);
		bindModeAvailability(SWISS_MODE_AVAILABILITY_NAME).to(SwissModeAvailability.class);
	}

	@Provides
	public SwissUtilityEstimator provideSwissUtilityEstimator(TripRouter tripRouter, Network network,
			ActivityFacilities facilities, CostModel costModel, TransitSchedule transitSchedule,
			SwissUtilityParameters parameters) {
		return new SwissUtilityEstimator(tripRouter, network, facilities, costModel, transitSchedule, parameters);
	}

	@Provides
	@Singleton
	public SwissUtilityParameters provideSwissUtilityParameters() throws IOException, ConfigurationException {
		SwissUtilityParameters parameters = SwissUtilityParameters.build(utilitySet, useRouteChoice, onlySignificant);

		if (cmd.hasOption("utility-parameters")) {
			File path = new File(cmd.getOptionStrict("utility-parameters"));
			SwissUtilityParameters.adaptFromFile(parameters, path);
		}

		SwissUtilityParameters.adaptFromCommandLine(parameters, cmd);
		return parameters;
	}

	@Provides
	@Singleton
	public CostModel provideCostModel(SwissUtilityParameters parameters) {
		return new CostModel(parameters.costs);
	}

	@Provides
	@Singleton
	public CostParameters provideCostParameters() {
		/*
		 * TODO: We can make this configurable from outside.
		 */

		return new CostParameters();
	}

	@Provides
	public SwissModeAvailability provideSwissModeAvailability(DiscreteModeChoiceConfigGroup dmcConfig) {
		ModeAvailabilityConfigGroup maConfig = dmcConfig.getCarModeAvailabilityConfig();
		return new SwissModeAvailability(maConfig.getAvailableModes());
	}
}
