package ch.ethz.matsim.projects.astra_2018_002.private_av;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.Scenario;
import org.matsim.api.core.v01.TransportMode;
import org.matsim.core.config.Config;
import org.matsim.core.config.groups.PlanCalcScoreConfigGroup.ModeParams;
import org.matsim.vehicles.VehicleType;

import ch.ethz.matsim.discrete_mode_choice.modules.config.DiscreteModeChoiceConfigGroup;

public class PrivateAVConfigurator {
	static final public Id<VehicleType> PRAV3_ID = Id.create("prav3", VehicleType.class);
	static final public Id<VehicleType> PRAV4_ID = Id.create("prav4", VehicleType.class);
	static final public Id<VehicleType> PRAV5_ID = Id.create("prav5", VehicleType.class);

	static final public String PRAV_3 = "prav3";
	static final public String PRAV_4 = "prav4";
	static final public String PRAV_5 = "prav5";

	static final public Collection<String> PRAV_MODES = Arrays.asList(PRAV_3, PRAV_4, PRAV_5);

	static public void configure(Config config) {
		PrivateAVConfigGroup privateConfig = (PrivateAVConfigGroup) config.getModules()
				.get(PrivateAVConfigGroup.GROUP_NAME);

		if (privateConfig == null) {
			privateConfig = new PrivateAVConfigGroup();
			config.addModule(privateConfig);
		}

		if (privateConfig.getUsePrivateAVs()) {
			DiscreteModeChoiceConfigGroup dmcConfig = (DiscreteModeChoiceConfigGroup) config.getModules()
					.get(DiscreteModeChoiceConfigGroup.GROUP_NAME);

			Set<String> cachedModes = new HashSet<>(dmcConfig.getCachedModes());
			cachedModes.addAll(PRAV_MODES);
			dmcConfig.setCachedModes(cachedModes);

			Set<String> restrictedModes = new HashSet<>(
					dmcConfig.getVehicleTourConstraintConfig().getRestrictedModes());
			restrictedModes.addAll(PRAV_MODES);
			dmcConfig.getVehicleTourConstraintConfig().setRestrictedModes(restrictedModes);

			Set<String> networkAndMainModes = new HashSet<>(Collections.singleton(TransportMode.car));
			networkAndMainModes.addAll(PRAV_MODES);

			config.plansCalcRoute().setNetworkModes(networkAndMainModes);
			config.qsim().setMainModes(networkAndMainModes);
			config.travelTimeCalculator().setAnalyzedModes(String.join(",", networkAndMainModes));
			config.travelTimeCalculator().setSeparateModes(false);

			for (String mode : PRAV_MODES) {
				ModeParams params = new ModeParams(mode);
				config.planCalcScore().addModeParams(params);
			}
		}
	}

	static public void assignPrivateAVs(Scenario scenario) {
		PrivateAVConfigGroup privateConfig = (PrivateAVConfigGroup) scenario.getConfig().getModules()
				.get(PrivateAVConfigGroup.GROUP_NAME);

		if (privateConfig.getUsePrivateAVs()) {
			AssignPrivatAVs.main(scenario);
		}
	}
}
