package ch.ethz.matsim.projects.astra_2018_002.shared_av.service_area;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import org.matsim.api.core.v01.Id;
import org.matsim.api.core.v01.network.Link;
import org.matsim.contrib.dvrp.data.Vehicle;

import com.google.inject.Inject;

import ch.ethz.matsim.av.config.AVGeneratorConfig;
import ch.ethz.matsim.av.data.AVVehicle;
import ch.ethz.matsim.av.generator.AVGenerator;
import ch.ethz.matsim.projects.astra_2018_002.shared_av.SharedAVConfigGroup;

public class ServiceAreaGenerator implements AVGenerator {
	private final String prefix;
	private final List<Link> links;
	private final int numberOfVehicles;
	private final Random random = new Random();
	private int numberOfGeneratedVehicles = 0;

	public ServiceAreaGenerator(String prefix, int numberOfVehicles, List<Link> links) {
		this.links = links;
		this.numberOfVehicles = numberOfVehicles;
		this.prefix = prefix;
	}

	@Override
	public boolean hasNext() {
		return numberOfGeneratedVehicles < numberOfVehicles;
	}

	@Override
	public AVVehicle next() {
		numberOfGeneratedVehicles++;

		// TODO: Here we create vehicles at random. Usually we also estimate the
		// population density at a certain link to get a better initial positioning. We
		// can add this here later (see the standard implementations in the AV package).

		Id<Vehicle> vehicleId = Id.create(prefix + "_" + String.valueOf(numberOfGeneratedVehicles), Vehicle.class);
		Link startLink = links.get(random.nextInt(links.size()));

		return new AVVehicle(vehicleId, startLink, 4.0, 0.0, Double.POSITIVE_INFINITY);
	}

	public static class Factory implements AVGeneratorFactory {
		@Inject
		ServiceArea serviceArea;

		@Inject
		SharedAVConfigGroup astraConfig;

		@Override
		public AVGenerator createGenerator(AVGeneratorConfig generatorConfig) {
			String operator = generatorConfig.getParent().getId().toString();

			String prefix = "av_" + operator;
			int numberOfVehicles = (int) generatorConfig.getNumberOfVehicles();

			return new ServiceAreaGenerator(prefix, numberOfVehicles, new ArrayList<>(serviceArea.getServedLinks()));
		}
	}
}
